"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var webpack = require("webpack");
var glob = require("glob");
var assign = require("deep-assign");
var UglifyJSPlugin = require('uglifyjs-webpack-plugin');
var ProgressPlugin = require('webpack/lib/ProgressPlugin');
var CleanWebpackPlugin = require('clean-webpack-plugin');
var CopyWebpackPlugin = require('copy-webpack-plugin');
var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
var SuppressChunksPlugin = require('suppress-chunks-webpack-plugin').default;
var WebpackNotifierPlugin = require('webpack-notifier');
var FriendlyErrorsWebpackPlugin = require('friendly-errors-webpack-plugin');
var NoEmitOnErrorsPlugin = webpack.NoEmitOnErrorsPlugin, WatchIgnorePlugin = webpack.WatchIgnorePlugin, EnvironmentPlugin = webpack.EnvironmentPlugin;
var CommonsChunkPlugin = webpack.optimize.CommonsChunkPlugin;
var nodeModules = path.join(process.cwd(), 'node_modules');
function flattenGlobPatterns(patterns, base) {
    if (base === void 0) { base = process.cwd(); }
    return patterns.reduce(function (prev, current) {
        prev.push.apply(prev, glob.sync(current, {
            cwd: base
        }));
        return prev;
    }, []);
}
function recurseGlob(patternObject, base) {
    if (base === void 0) { base = process.cwd(); }
    Object.keys(patternObject).forEach(function (key) {
        if (typeof patternObject[key] === 'string') {
            patternObject[key] = glob.sync(patternObject[key], {
                cwd: base
            });
        }
        else {
            patternObject[key] = recurseGlob(patternObject[key], base);
        }
    });
    return patternObject;
}
function commonConf(config, option) {
    var BASE_PATH = path.join(process.cwd(), config.app.root);
    var entries = assign({
        vendor: flattenGlobPatterns(config.app.vendor, BASE_PATH)
    }, recurseGlob(config.app.entry, BASE_PATH));
    var excludeChunks = [];
    if (option.extractCss) {
        for (var key in config.app.entry) {
            var paths = config.app.entry[key];
            if (typeof paths === 'string') {
                var ext = path.extname(paths);
                if (ext === '.css' || ext === '.scss') {
                    excludeChunks.push(key);
                }
            }
            else {
                var isAllStyles = paths.every(function (filepath) {
                    var ext = path.extname(filepath);
                    return (ext === '.css' || ext === '.scss');
                });
                if (isAllStyles) {
                    excludeChunks.push(key);
                }
            }
        }
    }
    var env = {
        production: option.production
    };
    if (option.environment && config.app.environments && config.app.environments[option.environment]) {
        env = require(path.resolve(process.cwd(), config.app.root, config.app.environments[option.environment]));
    }
    var copyConf = config.app.assets.map(function (asset) {
        if (typeof asset === 'string') {
            return {
                from: asset,
                to: asset
            };
        }
        else if (typeof asset === 'object') {
            return {
                context: asset.input,
                from: asset.glob,
                to: asset.output
            };
        }
        else {
            console.error('unknown type', asset);
        }
    });
    return {
        context: path.resolve(process.cwd(), config.app.root),
        entry: entries,
        output: {
            pathinfo: true,
            path: path.resolve(process.cwd(), config.app.outDir),
            filename: 'assets/js/[name].bundle.js',
            chunkFilename: '[id].chunk.js',
            publicPath: '/',
            jsonpFunction: 'frp',
            library: '[name]_library'
        },
        module: {
            rules: [
                {
                    test: /\.json$/,
                    loader: 'json-loader'
                },
                {
                    test: /\.(eot|svg)$/,
                    loader: 'file-loader?name=[name].[hash:20].[ext]'
                },
                {
                    test: /\.(jpg|png|gif|otf|ttf|woff|woff2|cur|ani)$/,
                    loader: 'url-loader?name=[name].[hash:20].[ext]&limit=10000'
                },
            ]
        },
        resolve: {
            modules: [
                path.resolve(process.cwd(), config.app.root),
                path.resolve(process.cwd(), 'node_modules'),
                'node_modules',
            ],
            extensions: [
                '.jsx',
                '.js',
                '.scss',
                '.css',
                '.ts',
                '.ejs',
                '.html',
                '.json'
            ],
        },
        resolveLoader: {
            modules: [
                path.resolve(process.cwd(), 'node_modules')
            ].concat(process.mainModule.paths)
        },
        devtool: (option.sourcemaps === true) ? '#source-map' : option.sourcemaps,
        target: 'web',
        stats: option.verbose ? 'verbose' : 'errors-only',
        plugins: (function () {
            var plugins = [
                new CommonsChunkPlugin({
                    name: 'vendor',
                    minChunks: function (module) { return module.resource && module.resource.startsWith(nodeModules); },
                    chunks: [
                        'main'
                    ]
                }),
                new EnvironmentPlugin(env),
                new NoEmitOnErrorsPlugin(),
                new CleanWebpackPlugin([
                    path.resolve(process.cwd(), config.app.outDir)
                ], {
                    verbose: option.verbose,
                    allowExternal: true
                }),
                new CopyWebpackPlugin(copyConf, {
                    debug: option.verbose ? 'debug' : 'warning'
                }),
                new ProgressPlugin(),
                new FriendlyErrorsWebpackPlugin(),
                new WebpackNotifierPlugin({ excludeWarnings: !option.verbose }),
                new SuppressChunksPlugin(excludeChunks, { filter: /\.js$/ })
            ];
            if (config.app.watchIgnore) {
                plugins.push(new WatchIgnorePlugin(config.app.watchIgnore));
            }
            if (option.production) {
                var uglifyOptions = config.app.scriptPreprocessorOptions ? config.app.scriptPreprocessorOptions.uglify || {} : {};
                plugins.push(new UglifyJSPlugin(uglifyOptions));
            }
            if (option.analyze) {
                plugins.push(new BundleAnalyzerPlugin({
                    openAnalyzer: true
                }));
            }
            return plugins;
        })(),
    };
}
exports.commonConf = commonConf;
